package in.slanglabs.assistants.retail;

import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import in.slanglabs.assistants.base.AssistantUIPosition;
import in.slanglabs.assistants.retail.FilterInfo;
import in.slanglabs.assistants.retail.Item;
import in.slanglabs.assistants.retail.SearchInfo;
import in.slanglabs.assistants.retail.SortingInfo;

public class SlangRetailAssistantUtils {

    private static String TAG = "SlangRetailAssistantUtils";

    private static Map<String,Object> mConfigOptions;

    static void setConfigOptions(Map<String,Object> configOptions) {
        mConfigOptions = configOptions;
    }

    static String getStringConfig(String key) {
        try {
            return mConfigOptions != null && mConfigOptions.containsKey(key) ?
                    (mConfigOptions.get(key) instanceof String ?
                            (String) mConfigOptions.get(key) : null) : null;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return null;
    }

    static boolean getBooleanConfig(String key) {
        try {
            return mConfigOptions != null && mConfigOptions.containsKey(key) ?
                    (mConfigOptions.get(key) instanceof Boolean ?
                            (boolean) mConfigOptions.get(key) : null) : null;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return false;
    }

    static List<String> getStringListConfig(String key) {
        if (mConfigOptions == null || !mConfigOptions.containsKey(key))
            return null;

        try {
            List<String> stringArray = (List<String>) mConfigOptions.get(key);
            if (null == stringArray || stringArray.size() == 0)
                return null;

            List<String> stringList = new ArrayList<>(stringArray.size());
            stringList.addAll(stringArray);
            return stringList;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
        }

        return null;
    }

    static AssistantUIPosition getAssistantUIConfig(String key) {
        AssistantUIPosition.BaseUIPosition uiPosition
                = AssistantUIPosition.BaseUIPosition.BOTTOM_CENTER;
        AssistantUIPosition assistantUIPosition
                = new AssistantUIPosition.Builder()
                .setBaseUIPosition(uiPosition)
                .setIsDraggable(false)
                .setIsForcedAtStartup(false)
                .build();

        if (mConfigOptions == null || !mConfigOptions.containsKey(key))
            return assistantUIPosition;

        try {
            HashMap<String,Object> assistantUIMap = (HashMap<String, Object>) mConfigOptions.get(key);
            if (null == assistantUIMap)
                return assistantUIPosition;
            String baseUIPosition = (String) assistantUIMap.get("baseUIPosition");
            switch (baseUIPosition != null ? baseUIPosition : "") {
                case "BOTTOM_LEFT":
                    uiPosition = AssistantUIPosition.BaseUIPosition.BOTTOM_LEFT;
                    break;
                case "CENTER_LEFT":
                    uiPosition = AssistantUIPosition.BaseUIPosition.CENTER_LEFT;
                    break;
                case "CENTER":
                    uiPosition = AssistantUIPosition.BaseUIPosition.CENTER;
                    break;
                case "CENTER_RIGHT":
                    uiPosition = AssistantUIPosition.BaseUIPosition.CENTER_RIGHT;
                    break;
                case "TOP_LEFT":
                    uiPosition = AssistantUIPosition.BaseUIPosition.TOP_LEFT;
                    break;
                case "TOP_CENTER":
                    uiPosition = AssistantUIPosition.BaseUIPosition.TOP_CENTER;
                    break;
                case "TOP_RIGHT":
                    uiPosition = AssistantUIPosition.BaseUIPosition.TOP_RIGHT;
                    break;
                case "BOTTOM_RIGHT":
                    uiPosition = AssistantUIPosition.BaseUIPosition.BOTTOM_RIGHT;
                    break;
                default:
                    uiPosition = AssistantUIPosition.BaseUIPosition.BOTTOM_CENTER;
            }
            int offsetX = assistantUIMap.containsKey("offsetX")?(int) assistantUIMap.get("offsetX"):-1;
            int offsetY = assistantUIMap.containsKey("offsetY")?(int) assistantUIMap.get("offsetY"):-1;
            boolean draggable = assistantUIMap.containsKey("draggable") && (boolean) assistantUIMap.get("draggable");
            boolean forcedAtStartup = assistantUIMap.containsKey("forcedAtStartup") && (boolean) assistantUIMap.get("forcedAtStartup");
            assistantUIPosition = new AssistantUIPosition.Builder()
                    .setBaseUIPosition(uiPosition)
                    .setOffsetX(offsetX)
                    .setOffsetY(offsetY)
                    .setIsDraggable(draggable)
                    .setIsForcedAtStartup(forcedAtStartup)
                    .build();
            return assistantUIPosition;
        } catch (Exception e) {
            Log.d(TAG, "mConfigOptions encountered an error");
            return assistantUIPosition;
        }
    }

    static Map<String, Object> mapSearchInfo(SearchInfo searchInfo) {
        Map<String, Object> searchInfoMap = new HashMap<>();
        if (searchInfo != null) {
            searchInfoMap.put("item", mapItem(searchInfo.getItem()));
            searchInfoMap.put("filterInfoList", mapFilterInfoList(searchInfo.getFilters()));
            searchInfoMap.put("sortingInfo", mapSortingInfo(searchInfo.getSorting()));
            searchInfoMap.put("isAddToCart", searchInfo.isAddToCart());
        }
        return searchInfoMap;
    }

    static Map<String, Object> mapItem(Item item) {
        Map<String, Object> itemMap = new HashMap<>();
        itemMap.put("id", item.getId());
        itemMap.put("brand", item.getBrand());
        itemMap.put("productType", item.getProductType());
        itemMap.put("description", item.getDescription());
        itemMap.put("category", item.getCategory());
        itemMap.put("variants", mapItemVariantList(item.getVariants()));
        itemMap.put("quantity", mapItemQuantity(item.getQuantity()));
        itemMap.put("size", mapItemSize(item.getSize()));
        itemMap.put("price", mapItemPrice(item.getPrice()));
        return itemMap;
    }

    static ArrayList<Object> mapFilterInfoList(List<FilterInfo> filterInfos) {
        ArrayList<Object> filterInfoListArray = new ArrayList<>();
        if (filterInfos != null) {
            for (FilterInfo filterInfo : filterInfos) {
                filterInfoListArray.add(mapFilterInfo(filterInfo));
            }
        }
        return filterInfoListArray;
    }

    private static Map<String, Object> mapSortingInfo(SortingInfo sortingInfo) {
        Map<String, Object> sortingInfoMap = new HashMap<>();
        if (sortingInfo != null) {
            sortingInfoMap.put("sortOrder", sortingInfo.getSortOrder().name());
            sortingInfoMap.put("sortType", sortingInfo.getSortType().name());
        }
        return sortingInfoMap;
    }

    static ArrayList<Object> mapItemVariantList(String[] variants) {
        ArrayList<Object> variantsListArray = new ArrayList<>();
        if (variants != null) {
            for (String variant : variants) {
                variantsListArray.add(variant);
            }
        }
        return variantsListArray;
    }

    static Map<String, Object> mapItemQuantity(Item.Quantity itemQuantity) {
        Map<String, Object> itemQuantityMap = new HashMap<>();
        if (itemQuantity != null) {
            itemQuantityMap.put("amount", itemQuantity.getAmount());
            itemQuantityMap.put("unit", itemQuantity.getUnit().name());
        }
        return itemQuantityMap;
    }

    static Map<String, Object> mapItemSize(Item.Size itemSize) {
        Map<String, Object> itemSizeMap = new HashMap<>();
        if (itemSize != null) {
            itemSizeMap.put("amount", itemSize.getAmount());
            itemSizeMap.put("unit", itemSize.getUnit().name());
        }
        return itemSizeMap;
    }

    static Map<String, Object> mapItemPrice(Item.Price itemPrice) {
        Map<String, Object> itemPriceMap = new HashMap<>();
        if (itemPrice != null) {
            itemPriceMap.put("minAmount", itemPrice.getMinAmount());
            itemPriceMap.put("maxAmount", itemPrice.getMaxAmount());
            itemPriceMap.put("currency", itemPrice.getCurrency().name());
        }
        return itemPriceMap;
    }

    private static Map<String, Object> mapFilterInfo(FilterInfo filterInfo) {
        Map<String, Object> filterInfoMap = new HashMap<>();
        if (filterInfo != null) {
            filterInfoMap.put("filterName", filterInfo.getFilterName());
            filterInfoMap.put("filterValue", filterInfo.getFilterValue());
        }
        return filterInfoMap;
    }

}
